const XLSX = require('xlsx');
const program = require('commander');
const fs = require('fs-extra');
const path = require('path');
const Preview = require("ro-crate").Preview;
const HtmlFile = require("ro-crate").HtmlFile;
const ROCrate = require("ro-crate").ROCrate;
const utf8 = require("utf8");

program
  .version("0.1.0")
  .description(
    "Extracts data from a spreadsheet to make an RO crate"
  )
  .arguments("<f>")
  .option("-c,  --cratescript [cratesript]", "URL of Crate-script directory")
  .option("-a, --additional-context [context]", "JSON file with context object")
  .option("-p, --places [places]", "Spreadsheet of places")
  .option("-d, --data [data]", "Data directory from which to copy image files")
  .action((f) => {file = f})
program.parse(process.argv);
const cratescript = program.cratescript;


const places = {};


function loadPlaces(file){
  var workbook = XLSX.readFile(file);
   const placeData = XLSX.utils.sheet_to_json(workbook.Sheets["Sheet1"]);
    //console.log(data);    
    for (let row of placeData) {
        //console.log(Object.keys(row));
        // Latitude	Longitude	Start date	End date	Label	Description
        var geo = {
            "@type": "Geocoordinates",
            "name": `Latitude: ${row["Latitude"]} Longitude: ${row["Longitude"]}`,
            "@id": `#${row["Latitude"]},${row["Longitude"]}`,
            "latitude": row["Latitude"],
            "longitude": row["Longitude"]
        }
        var place = {
            "@id": `#place_${row["Place"]}`,
            "@type": "Place",
            "name": row["Place"],
            "geo": {"@id": geo["@id"]},
            "startDate": row["Start date"],
            "endDate": row["End date"],
            "description": row["Description"],
            "note": row["note"]
        }
        crate.getGraph().push(place);
        crate.getGraph().push(geo);
        places[place.name] = place;
    }
}

const crate = new ROCrate();
crate.index();
const root = crate.getRootDataset();
if (program.additionalContext) {
    const additionalContext = JSON.parse(fs.readFileSync(program.additionalContext));
    const vocab = additionalContext["@vocab"];
    delete additionalContext["@vocab"];
    const newContext = {};
    for (key of Object.keys(additionalContext)) {
        const vocabURL = `${vocab}${key}`
        const id = `http://purl.org/adhoc?@type=rdf:Property&@vocab=${vocab}&rdfs:label=${key}`
        const url = `${id}&rdfs:comment=${additionalContext[key]["description"]}&disambiguatingDescription=${additionalContext[key]["disambiguatingDescription"]}&@id=${encodeURIComponent(id)}`;
        const sameAs = {
            "@id": vocabURL,
            "sameAs": url
        }
        const additionalProp = {
            "@id": id,
            "@type": "rdf:Propery",
            "rdfs:label": key,  
            "rdf:comment": additionalContext[key],
            "@vocab": vocab
        }
        crate.getGraph().push(additionalProp)
        crate.getGraph().push(sameAs);
        newContext[key] = url;

    }
    crate.json_ld["@context"].push(newContext);
}
root.name = "Convictions Sample Data DEMO"
root.hasPart = [];


function getPlace(name) {
    if (!places[name]) {
        places[name] = {
            "@id": `#place_${name}`,
            "@type": "Place",
            "name": name
        }
        crate.getGraph().push(places[name]);

    }
    return places[name];
}

main(file, cratescript);

function findFiles(dataDir, files) {
    for (let f of fs.readdirSync(dataDir)) {
        var p = path.join(dataDir, f)
        stats = fs.lstatSync(p);
       if (stats.isDirectory()) {
            findFiles(p, files)
        }
        files[f.replace(/\..*$/, "")] = p;
        files[f.replace(/_0+/g, "_").replace(/\..*$/, "")] = p;
        
        
    }
}

async function main(file, cratescript) {
    if (program.places) {
        loadPlaces(program.places)
    }
    const destDir = path.dirname(file);
    const dataDir = program.data;
    var fileMap = {};
    if (dataDir) {
        findFiles(dataDir, fileMap);
    }
   //console.log(fileMap);
  
    var workbook = XLSX.readFile(file);
    const data = XLSX.utils.sheet_to_json(workbook.Sheets["Sheet1"]);
    //console.log(data);
    var count = 0;
    
    for (let row of data) {
        //console.log(Object.keys(row));
        const names = row["Name (and aliases)"]
                          .split(/;\s*/)
        //console.log(names);
        var birthplace = getPlace(row.Birthplace);
        const mainName = names.shift();
        var person = {
            "@id": `#person_${count}`,
            "@type": "Person",
            "name": mainName,
            "alternateName": names.map((n)=>{return n.replace(/^./,"").replace(/.$/, "")}),
            "birthDate": row["Birth year"].toString(),
            "birthPlace": {"@id": birthplace["@id"]},
            "religion": row["Religion"],
            "education": row["Education"]
        };

        count += 1;
        for (let c of row["Conviction History"].split(";")) {
            const conv = c.split(/,\s*/);
   
            convictionPlace = getPlace(conv[3]);
            //6 MONTHS, 2-NOV-1904, VAGRANCY, MELBOURNE PETTY SESSIONS;
            if(conv[1]){
                const date = conv[1].toString();

                var action = {
                    "@id": `#action_${count++}`,
                    "@type": ["Action"],
                    "object": {"@id": person["@id"]},
                    "name": `${date} ${mainName}: ${conv[0]} ${conv[2]} ${convictionPlace.name}`,
                    "sentence" : conv[0],
                    "offence": conv[2],
                    "startTime": date,
                    "location": {"@id": convictionPlace["@id"]}
                }
                crate.getGraph().push(action);
        }
        }
        var filename = row["Image ID"];
        if (fileMap[filename]) {
            fileNameWithExt = path.basename(fileMap[filename]);
            const targetPath = path.join(destDir, fileNameWithExt);
            if (!fs.existsSync(targetPath)){
             console.log("Copying", targetPath);
             fs.copyFileSync(fileMap[filename], targetPath);
             }
             filename = fileNameWithExt;
         
        } else {
            console.log("Not found", filename);
        }
        var file = {
            "name": `${mainName}: record`,
            "@id": filename,
            "@type": ["File", "CreativeWork"],
            "about": {"@id": person["@id"]}
        };

        crate.getGraph().push(person);
        crate.getGraph().push(file);
        root.hasPart.push({"@id": file["@id"]});

        
    }

    fs.writeFileSync(path.join(destDir, "ro-crate-metadata.json"), JSON.stringify(crate.json_ld, null, 2));
    const preview = await new Preview(crate);
    const f = new HtmlFile(preview);
    const html = path.join(destDir, "ro-crate-preview.html");
    fs.writeFileSync(html, await f.render(cratescript));
    //console.log("Conviction", conviction);
}



